/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.dao;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import metridoc.component.sqlplus.StreamingJdbcTemplate;
import metridoc.impl.iterator.SqlIterator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;

/**
 * 
 * @author Narine Ghochikyan
 *
 */
public class SpringSqlPlusDao implements SqlPlusDao {
	private SimpleJdbcTemplate jdbcTemplate;
	private DataSource dataSource;
	
	private static final Logger LOG = LoggerFactory.getLogger(SpringSqlPlusDao.class);
	
	public SpringSqlPlusDao(DataSource dataSource){
		this.dataSource = dataSource;
		jdbcTemplate = new SimpleJdbcTemplate(dataSource);
	}

	@Override
	public int update(String[] queries) {
		int totalRows = 0;
		for (String query : queries) {
			query = query.trim();
			if(!query.equals("")){
				int rowsAffected = jdbcTemplate.update(query);	
				LOG.info("Query :: " + query + " :: " + rowsAffected + " rows affected");
				totalRows += rowsAffected;
			}
		}
		return totalRows;
	}

	@Override
	public SqlRowSet queryForRowSet(String query, int fetchSize) {
		JdbcTemplate jdbcTemplate = new StreamingJdbcTemplate();
		jdbcTemplate.setDataSource(dataSource);
		jdbcTemplate.setFetchSize(fetchSize);
		if(!query.startsWith("select")){
			query = "select * from " + query;
		}
		return jdbcTemplate.queryForRowSet(query);
	}

	@Override
	public Object batchUpdate(List values, String sql, boolean detailedOutput) {
		 SqlParameterSource[] batch;

         if (values.get(0) instanceof Map) {
             Map[] arrayForTransfer = new Map[values.size()];
             values.toArray(arrayForTransfer);
             batch = SqlParameterSourceUtils.createBatch(arrayForTransfer);
         } else {
             batch = SqlParameterSourceUtils.createBatch(values.toArray());
         }

         int[] updateCounts = jdbcTemplate.batchUpdate(sql, batch);
         return detailedOutput ? updateCounts : getSum(updateCounts);
	}

	@Override
	public Object batchUpdate(SqlRowSet sqlRowSet, String sql, int batchSize, boolean detailedOutput) {
	      SqlIterator iterator = new SqlIterator(sqlRowSet);
          List<Map<String, Object>> values = new ArrayList<Map<String, Object>>();
          List<int[]> updateCountsList = new ArrayList<int[]>();
          int updateCountsFlatSize = 0;
          Object updateCounts = null;
          int updateCountsSum = 0;
          while (iterator.hasNext()) {
              if (values.size() == batchSize) {
                  updateCounts = batchUpdate(values, sql, detailedOutput);
                  if (detailedOutput) {
                      updateCountsFlatSize += ((int[]) updateCounts).length;
                      updateCountsList.add((int[]) updateCounts);
                  } else {
                      updateCountsSum += (Integer) updateCounts;
                  }
                  values.clear();
              }
              values.add((Map<String, Object>) iterator.next());
          }

          if(values.size() > 0){
          	updateCounts = batchUpdate(values, sql, detailedOutput);
	            if (detailedOutput) {
	                updateCountsFlatSize += ((int[]) updateCounts).length;
	                updateCountsList.add((int[]) updateCounts);
	            } else {
	                updateCountsSum += (Integer) updateCounts;
	            }
          }
	        return detailedOutput ? toFlatArray(updateCountsList,
	                    updateCountsFlatSize) : updateCountsSum;
	}
	
	 private int getSum(int[] array) {
         int result = 0;
         for (int i = 0; i < array.length; i++) {
             result += array[i];
         }
         return result;
     }

     private int[] toFlatArray(List<int[]> list, int flatArraySize) {
         int[] result = new int[flatArraySize];
         int[] array;
         int k = 0;
         for (int i = 0; i < list.size(); i++) {
             array = list.get(i);
             for (int j = 0; j < array.length; j++) {
                 result[k] = array[j];
                 k++;
             }
         }
         return result;
     }
}
